import json
from sqlalchemy import create_engine
import pandas as pd


class SnowflakeDatabase:
    def __init__(self, config):
        self.config = config

        self.snowflake_engine = create_engine(
            f"snowflake://{self.config.snowflake_username}:{self.config.snowflake_password}"
            f"@{self.config.snowflake_account}/"
            f"{self.config.snowflake_database}&role={self.config.snowflake_role}"
        )

    def get_data(self, query):
        """
            param
                query: SQL Query with schema expected
            return: A pandas dataframe of the SQL query
        """
        conn = self.snowflake_engine.connect()
        print("Connected to Snowflake database")
        data = pd.read_sql(sql=query, con=conn)
        print("Query ran Successfully")
        return data

    def execute_query(self, query):
        """
            query: Provide a SQL Query in a string format
            Return: Success or Fail message
        """
        # ctx = SnowflakeHook.get_cursor()
        try:
            conn = self.snowflake_engine.connect()
            query_list = query.split(";")[:-1]
            for query_internal in query_list:
                try:
                    conn.execute(query_internal)
                except Exception as e:
                    print(e)
                    raise ValueError("Error executing snowflake query: " + str(e))
        except Exception as err:
            error_code = f"snowflake connection error: failed with Error: {err}"
            raise ValueError(error_code)
        else:
            self.snowflake_engine.dispose()

    def insert_step_metadata(self, step_start_date, step_start_time, process_name, user_name, step_name,
                             step_end_time="null", time_taken="null",
                             status='Running', notes=json.dumps("None")):
        """
        Update metadata table with a success for the particular step within the process
         Args:
        process_name: Name of the process
        step_name: Name of the Step withing the process
        execution_time: Time when the step in the process started
        user_name: User who executed the process/step
        process_end_time: Time at which the step within the process finished
        time_taken: time taken for the step to finish
        error: Error seen in the step/process
        metadata_table: table where metadata will be stored

        Returns: None. Sends out notification and updates the metadata table within Snowflake with the error data.
        """
        metadata_table = "ETL_PROCESS_MONITOR"
        query = (
                f'INSERT  INTO "{self.config.snowflake_database}"."MKT_SANDBOX"."{metadata_table}"'
                f"(select '%s','%s','%s','%s','%s',NULLIF('%s','null'),NULLIF('%s','null'),'%s',PARSE_JSON('%s'));"
                % (step_start_date, step_start_time, str(process_name), str(user_name), str(step_name), step_end_time,
                   time_taken, str(status), notes)
        )
        self.execute_query(query)

    def update_step_metadata(self, step_start_date, step_start_time, process_name, step_name, status,
                             step_end_time="null", time_taken="null", notes=json.dumps("None")):
        """
            :param Job_name: Function Name
            :param status: Success/Failure
            :param e: Exception message to be inserted
            """
        metadata_table = "ETL_PROCESS_MONITOR"
        query = (
            f'update "{self.config.snowflake_database}"."MKT_SANDBOX"."{metadata_table}"'
            f"set STEP_END_TIME = NULLIF('{step_end_time}','null'), status = '{status}', "
            f"time_taken = NULLIF('{time_taken}','null'), NOTES = PARSE_JSON('{notes}') where STEP_NAME = '{step_name}' and "
            f" STEP_START_DATE = '{step_start_date}' and  STEP_START_TIME = '{step_start_time}' and process_name = '{process_name}';"
            f"commit;"
        )
        self.execute_query(query)

    def insert_process_metadata(self, process_start_date, process_start_time, process_name, process_end_time='null',
                                time_taken='null', status='Running', notes=json.dumps("None")):
        """
            : param process_start_date: Process start date
            : param process_start_time: Process start date and time
            : param process_name: process Name
            : param process_end_time process_end_time
            : param status: Success/Failure
            : param e: Exception message to be inserted
        """
        metadata_table = "ETL_PROCESS_METADATA"
        query = (
                f'INSERT  INTO "{self.config.snowflake_database}"."MKT_SANDBOX".{metadata_table}'
                f"(select '%s','%s','%s',NULLIF('%s','null'),NULLIF('%s','null'),'%s',PARSE_JSON('%s'));"
                % (
                    process_start_date, process_start_time, str(process_name), process_end_time,
                    time_taken, str(status), notes)
        )
        self.execute_query(query)

    def update_process_metadata(self, process_start_date, process_start_time, process_name, status,
                                process_end_time="null", time_taken="null", notes=json.dumps("None")):
        """
            : param process_start_date: Process start date
            : param process_start_time: Process start date and time
            : param process_name: process Name
            : param process_end_time process_end_time
            : param status: Success/Failure
            : param e: Exception message to be inserted
        """
        metadata_table = "ETL_PROCESS_METADATA"
        query = (
            f'update "{self.config.snowflake_database}"."MKT_SANDBOX"."{metadata_table}"'
            f"set PROCESS_END_TIME = NULLIF('{process_end_time}','null'), status = '{status}', "
            f"time_taken = NULLIF('{time_taken}','null') , NOTES = PARSE_JSON('{notes}') where process_name = '{process_name}' and "
            f" PROCESS_START_DATE = '{process_start_date}' and  PROCESS_START_TIME = '{process_start_time}';commit;"
        )
        self.execute_query(query)

    def get_scalar_data(self, query):
        """
        : param query: A scalar subquery which returns a single value
        : return: Scalar subquery results
        """
        cur = self.snowflake_engine.connect()
        print("Connected to Snowflake database")
        result = cur.execute(query).scalar()
        print("Query ran Successfully")
        cur.close()
        return result













